% On Particle Methods for Parameter Estimation in General
%                   State-Space Models
% This code uses the N^2 method of  Section 2.3.2 p.7 to compute 
% the smoothed additive functionals in the examples of
% Section 5.1
% linear Gaussian model
% X_{n+1} = phi*X_n+ tau*V_{n+1},   v_{n+1} ~ N(0,1) 
% Y_n = X_n + sigma*W_n,  W_n ~ N(0,1)
%
% This program computes sufficent statistics only
%
% Authors: S.S. Singh, N. Kantas, A. Doucet, N. Chopin
% November 2011



clear all

rand('twister', sum(100*clock));
randn('state',sum(100*clock));

tic


GetData;
% load simulated_y.mat YY Y0




obs=YY;

Nobs=10000;

N=100; % no of particles


% Declare the true parameters of the model
phi=.8;
sigma_v =sqrt(0.1);
c=1;
sigma_w=1;
sigma_0=1;





% define the log-likelihood function
log_g=inline('-log(sigma) - 0.5*y.^2/(sigma^2)','y','sigma');


% define the log of the transition density
log_f=inline('-0.5*log(2*pi) - log(sigma) -0.5*(xn-phi*xn_1).^2 / sigma^2','xn', 'xn_1', 'phi','sigma');

% define sufficient statistics
s=inline('[xn_1.^2; xn_1; xn_1.*xn ]'  ,'xn_1','xn','yn');



MCruns=13;
S_history_1=zeros(Nobs,MCruns);
S_history_2=zeros(Nobs,MCruns);
S_history_3=zeros(Nobs,MCruns);

for ii=1:MCruns

    T=zeros(3,N);
    Tu=T;   % stores the updated T

    % generating particles for time 0
    weights=ones(N,1)/N;
    particles=randn(N,1)* sigma_0;  % sampling from the prior

    weights=exp(log_g(  obs(1) - c*particles,sigma_w )  );
    weights=weights/sum(weights);


    %wbarHandle=waitbar(0,[num2str(Nobs) ' obs: run ' num2str(ii) '/' num2str(MCruns)]);

    for i=2:Nobs
        %  waitbar(i/Nobs,wbarHandle);


        particlesPrev=particles;
        weightsPrev=weights;

        % resample the particles
        particles=SysResample(particlesPrev,weightsPrev,N);


        % predict the particles
        particles=particles*phi+sigma_v*randn(N,1);

        % correct the weights
        weights=exp(log_g(  obs(i) - c*particles,sigma_w )  );
        weights=weights/sum(weights);



        for j=1:N

            temp=s(particlesPrev',particles(j)*ones(1,N),obs(i)) + T;
            temp2=weightsPrev'.* exp(log_f(particles(j),particlesPrev',phi,sigma_v   ));
            Tu(:,j)=temp*temp2';
            Tu(:,j)=Tu(:,j)/sum(temp2);
        end

        T=Tu;

        S=Tu*weights;
        S_history_1(i,ii)=S(1);
        S_history_2(i,ii)=S(2);
        S_history_3(i,ii)=S(3);

    end

end

simtime=toc
save suf_stat_N2.mat
