function [x] = SysResample(xu,q,N)

% FUNCTION : Performs the systematic resampling step of the SIR algorithm
%           in order(numSamples) steps.
% INPUTS  : - xu = Particles that are to be resampled. Each row is a
% particle. This matrix can be arbitrary dimension
% 
%           - q = Normalised importance ratios.
%           - N = number of samples required
% OUTPUTS : - x = Resampled state samples.

% AUTHOR  : Nick Whiteley based on algorithm in 'Beyond the Kalman Filter'
            % Ristic et al.
% DATE    : 9/1/2006



[dummy, numVars]=size(xu);  % N = number of samples;
x = zeros(N, numVars);

c = zeros(N,1);
u = zeros(N,1);

c = cumsum(q);
i = 1;

%draw starting point
u(1) = rand/N;

for j=1:N

    u(j)=u(1)+(j-1)/N;
    
    while u(j)>c(i)
        i = i+1;
    end;
    
    x(j,:) = xu(i,:);
 
end;



