% On Particle Methods for Parameter Estimation in General
%                   State-Space Models
% Online em
% This code uses the N method in Section 8.3 of Cappé, O., Moulines, É. and Rydén, T. (2005). 
% Inference in Hidden Markov Models. New York:Springer-Verlag.
% to compute the smoothed additive functionals in the examples of Section 5.1
% linear Gaussian model
% X_{n+1} = phi*X_n+ tau*V_{n+1},   v_{n+1} ~ N(0,1) 
% Y_n = X_n + sigma*W_n,  W_n ~ N(0,1)
% 
% Authors: S.S. Singh, N. Kantas, A. Doucet, N. Chopin
% November 2011

clear all
rand('twister', sum(100*clock));
randn('state',sum(100*clock));

tic

N=150^2; % no of particles
iter=1e5; % no of online parameter estimates
burnin=5e3;
MCruns=12;


% define the log-likelihood function
log_g=inline('-0.5*log(2*pi)-log(sigma) - 0.5*(y.^2)/(sigma^2)','y','sigma');

% define the log of the transition density
log_f=inline('-0.5*log(2*pi) - log(sigma) -0.5*(xn-phi*xn_1).^2 / (sigma^2)','xn', 'xn_1', 'phi','sigma');

% % % cdefine sufficient statistics
s=inline('[(yn-xn).^2; xn_1.^2; xn_1.*xn; xn.^2]'  ,'xn_1','xn','yn');

% define argmax function of M-step
lambda=inline('[z3/z2; z4-((z3^2)/z2); z1]','z1','z2','z3','z4');


GetData;

PhiHistAll=zeros(MCruns,iter);
Tau2HistAll=zeros(MCruns,iter);
Sigma2HistAll=zeros(MCruns,iter);


S1=zeros(MCruns,iter);
S2=zeros(MCruns,iter);
S3=zeros(MCruns,iter);
S4=zeros(MCruns,iter);

thetaInit=[.1,.1,.2]';

for ll=1:MCruns

    theta=thetaInit;

    T=zeros(4,N);
    Tu=T;   % stores the updated T

    % % generating particles for time 0
    particles= sqrt(theta(2))*rand(N,1);

    weights=exp(log_g(  Y0-particles, sqrt( theta(3) )));
    weights=weights/sum(weights);

    thetaHist=zeros(3,iter);
    estHist=zeros(4,iter);

    for i=1:iter

        gam=iter^-.75;
        Y=YY(i);


        tempParticlesAndT=SysResample([particles,T'],weights,N);
        particlesPrev=tempParticlesAndT(:,1);
        T=tempParticlesAndT(:,2:end)';


        % NOTE theta =[phi, tau^2, sigma^2];

        % predict the particles
        particles=particlesPrev*theta(1)+sqrt(theta(2))*randn(N,1);

        % correct the weights
        weights=exp(log_g(  Y-particles, sqrt( theta(3)  )  ));
        weights=weights/sum(weights);


        Tu=gam*s(particlesPrev',particles',Y) + (1-gam)*T;

        S=Tu*weights;

        estHist(:,i)=S;
        thetaHist(:,i)=theta;

        theta=lambda(S(1),S(2),S(3),S(4));  % update theta

        if theta(1)>1 || theta(1)<-1 || theta(2) <0 || theta(3)<0
            theta=thetaHist(:,i);
            disp('some variance negative check pls')
        end
        
%         disp([num2str(i) ' : ' num2str(theta')])
        
        if i<burnin
            theta=thetaInit;
        end
        theta(3)=sigma_true^2;

        T=Tu;


    end

    PhiHistAll(ll,:)=thetaHist(1,:);
    Tau2HistAll(ll,:)=thetaHist(2,:);
    Sigma2HistAll(ll,:)=thetaHist(3,:);


    S1(ll,:)=estHist(1,:);
    S2(ll,:)=estHist(2,:);
    S3(ll,:)=estHist(3,:);
    S4(ll,:)=estHist(4,:);

    if mod(ll,1)==0
        save long10runs_N100_1.mat
    end
end
simtime=toc;

save long10runs_N100_1.mat

