function [x q]=pf(N,tau2SampleNew,sigma2SampleNew,y,m0,C0);

% On Particle Methods for Parameter Estimation in General
%                   State-Space Models
% simple particle filter implementation od 
% linear Gaussian model
% X_{n+1} = phi*X_n+ tau*V_{n+1},   v_{n+1} ~ N(0,1) , X_0 ~ N(m0,C0)
% Y_n = X_n + sigma*W_n,  W_n ~ N(0,1)
%
% This program is used within PMMH
% INPUTS: N number of particles
%         tau2SampleNew, sigma2SampleNew are current estimates/samples of tau^2 and sigma^2
%         y dataset
%         m0, C0 see model above
% OUTPUTS: x current sample of X_{0:T} after last resampling step
%          q unnormalised weights from time 1 to T
% Authors: N. Kantas, A. Doucet, S.S. Singh, N. Chopin
% November 2011


T=size(y,1);
if T==1
    T=size(y,2);
    disp('check size of y')
end
    
a=1;
b=sqrt(tau2SampleNew);
c=1;
d=sqrt(sigma2SampleNew);

% particle approx
x=zeros(N,T+1);
% unnormalised weights
q=zeros(N,T+1);

for t=1:T+1
    if t==1
        x(:,t) = m0+sqrt(C0)*randn(N,1);
    else
        noise = b*randn(N,1);
        x(:,t) = a*x(:,t-1)+noise;
    end

    if t~=1
        m = c*x(:,t);

        q(:,t) = exp(-.5*(d^(-2))*(y(t-1)*ones(size(m))-m).^(2))/(2*pi*d);

        w = q(:,t)/sum(q(:,t));

        index= multinomialR([1:N]',w);

        x(:,t)=x(index,t);
    end
end;

q=q(:,2:end);

return