% On Particle Methods for Parameter Estimation in General
%                   State-Space Models
% Particle MCMC
% Code to implement PMMH for
% linear Gaussian model
% X_{n+1} = phi*X_n+ tau*V_{n+1},   v_{n+1} ~ N(0,1) 
% Y_n = X_n + sigma*W_n,  W_n ~ N(0,1)
%
% 
% Authors:  N. Kantas, A. Doucet, S.S. Singh, N. Chopin
% November 2011


clear

rand('twister', sum(100*clock));
randn('state',sum(100*clock));

tic


N = 500;             % Number of particles;

T=500;


GetData;


% Declare priors
m0=0; C0=10; % x0 is N(m0,C0)
a0=1; b0=1; % sigma2 is IG(a0,b0)
c0=1; d0=1; % tau2 is IG(c0,d0)

% NOTE on inline for simplicity assume a0=c0 and b0=d0
inverse_gamma_unnormalised=inline('(x^(-a-1))*exp(-(b/x))','x','a','b');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% IMPUTE FIRST SAMPLE by sampling from the prior

sigma2Sample =1./ gamrnd(a0,1/b0,1,1);
tau2Sample=1./ gamrnd(c0,1/d0,1,1);



% product of un-normalised priors
Prior=inverse_gamma_unnormalised(sigma2Sample,a0,b0)*...
    inverse_gamma_unnormalised(tau2Sample,c0,d0);
% particle filter
[x q]=pf(N,tau2Sample,sigma2Sample,y,m0,C0);
% log posterior
logYsum=sum(log(mean(q,1)))+log(Prior);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Start Metropolis Hastings

rep=100000;

SIGMA2=zeros(rep,1);
TAU2=zeros(rep,1);
LOGY=TAU2;
RATIO=[];

ll=1;
while ll<=rep

    % Random Walk move with Gaussian
    % Set initial condition to true guy   xSampleNew=xSample+sqrt(0.05)*randn(1,1);

    rw_step=.07;

    sigma2SampleNew = sigma2Sample+rw_step*randn(1,1);
    tau2SampleNew = tau2Sample+rw_step*randn(1,1);


    while sigma2SampleNew <=0
        sigma2SampleNew = sigma2Sample+rw_step*randn(1,1);
    end
    while tau2SampleNew<=0
        tau2SampleNew = tau2Sample+rw_step*randn(1,1);
    end

    % New product of un-normalised priors
    PriorNew=inverse_gamma_unnormalised(sigma2SampleNew,a0,b0)*...
        inverse_gamma_unnormalised(tau2SampleNew,c0,d0);

    % % % % % % % % % % % % % %

    [x q]=pf(N,tau2SampleNew,sigma2SampleNew,y,m0,C0);

    logYsumNew=sum(log(mean(q,1)))+log(PriorNew);
    % % % % % % % % % %

    ratio=exp(logYsumNew-logYsum);

    accept_ratio=min(1,ratio);

    RATIO=[RATIO accept_ratio];

    u=rand(1);
    if u<=accept_ratio && ~isnan(ratio) && logYsumNew~=0
        SIGMA2(ll,1)=sigma2SampleNew;
        TAU2(ll,1)=tau2SampleNew;
        LOGY(ll,1)=logYsumNew;

        logYsum=logYsumNew;

        sigma2Sample=sigma2SampleNew;
        tau2Sample=tau2SampleNew;

    else

        SIGMA2(ll,1)=sigma2Sample;
        TAU2(ll,1)=tau2Sample;
        LOGY(ll,1)=logYsum;


    end

    ll=ll+1;
    if mod(ll,10000)==0
        disp(['completed iteration ' num2str(ll) ' .... after ' num2str(size(RATIO,2)) ' attempts .......'])
        save pmcmc_run_21_2.mat
    end

end

simtime=toc

save pmcmc_run_21_2.mat
exit


